@@ -403,7 +403,7 @@ void CViewer3dDoc::OnUpdateOverlappedBox(CCmdUI* pCmdUI)
 
 void CViewer3dDoc::OnObjectRemove()
 {
-  // A small trick to avoid complier error (C2668).
+  // A small trick to avoid compiler error (C2668).
   const Handle(AIS_InteractiveObject)& aBox = myBox;
   if(myAISContext->IsSelected (aBox))
     myBox.Nullify();
@@ -506,7 +506,7 @@ void CViewer3dDoc::InputEvent(const Standard_Integer /*x*/,
 
   Quantity_Color CSFColor;
   COLORREF MSColor;
-  myAISContext->Select (Standard_True);
+  myAISContext->SelectDetected();
 
   // Change the color of a selected face in a user cylinder
   if (myState == FACE_COLOR)
@@ -1020,8 +1020,7 @@ void CViewer3dDoc::OnDumpView()
     pView->UpdateWindow();
   }
 
-  myViewer->InitActiveViews();
-  Handle(V3d_View) aView = myViewer->ActiveView();
+  Handle(V3d_View) aView = myViewer->ActiveViews().First();
   ExportView (aView);
 }
 
