@@ -11,7 +11,7 @@ IMPLEMENT_STANDARD_RTTIEXT(User_Cylinder,AIS_InteractiveObject)
 
 #include <Graphic3d_ArrayOfTriangles.hxx>
 #include <Graphic3d_StructureManager.hxx>
-#include <PrsMgr_PresentationManager3d.hxx>
+#include <PrsMgr_PresentationManager.hxx>
 #include <StdPrs_ToolTriangulatedShape.hxx>
 #include <Poly_Connect.hxx>
 #include <TColgp_Array1OfPnt.hxx>
@@ -20,7 +20,7 @@ IMPLEMENT_STANDARD_RTTIEXT(User_Cylinder,AIS_InteractiveObject)
 #include <GProp_PGProps.hxx>
 #include <Quantity_Color.hxx>
 #include <BRepMesh_IncrementalMesh.hxx>
-#include <Prs3d.hxx>
+#include <StdPrs_ToolTriangulatedShape.hxx>
 
 #include <AIS_GraphicTool.hxx>
 
@@ -28,7 +28,7 @@ IMPLEMENT_STANDARD_RTTIEXT(User_Cylinder,AIS_InteractiveObject)
 //
 
 User_Cylinder::User_Cylinder(const Standard_Real R, const Standard_Real H) :
-AIS_InteractiveObject(PrsMgr_TOP_ProjectorDependant)
+AIS_InteractiveObject(PrsMgr_TOP_ProjectorDependent)
 {
   BRepPrimAPI_MakeCylinder S(R,H);
   myShape = S.Shape();
@@ -38,7 +38,7 @@ AIS_InteractiveObject(PrsMgr_TOP_ProjectorDependant)
 }
 
 User_Cylinder::User_Cylinder(const gp_Ax2 CylAx2, const Standard_Real R, const Standard_Real H) :
-AIS_InteractiveObject(PrsMgr_TOP_ProjectorDependant)
+AIS_InteractiveObject(PrsMgr_TOP_ProjectorDependent)
 
 {
   BRepPrimAPI_MakeCylinder S(CylAx2,R,H);
@@ -49,7 +49,7 @@ AIS_InteractiveObject(PrsMgr_TOP_ProjectorDependant)
   myColor = Quantity_NOC_KHAKI4;
 }
 
-void User_Cylinder::Compute(const Handle(PrsMgr_PresentationManager3d)& /*aPresentationManager*/,
+void User_Cylinder::Compute(const Handle(PrsMgr_PresentationManager)& /*aPresentationManager*/,
                             const Handle(Prs3d_Presentation)& aPresentation,
                             const Standard_Integer aMode ) 
 {
@@ -76,7 +76,7 @@ case 6: //color
 
     Handle(Graphic3d_StructureManager) aStrucMana = GetContext()->MainPrsMgr()->StructureManager();
 
-    Handle(Graphic3d_Group) mygroup = Prs3d_Root::CurrentGroup(aPresentation);
+    Handle(Graphic3d_Group) mygroup = aPresentation->CurrentGroup();
     myAspect = (new Prs3d_ShadingAspect())->Aspect();
     Graphic3d_MaterialAspect material = myAspect->FrontMaterial();
     material.SetAmbientColor (Quantity_NOC_BLACK);
@@ -88,7 +88,7 @@ case 6: //color
     mygroup->SetPrimitivesAspect(myAspect);
     myAspect->SetEdgeOn();
 
-    myDeflection = Prs3d::GetDeflection(myShape,myDrawer);
+    myDeflection = StdPrs_ToolTriangulatedShape::GetDeflection(myShape,myDrawer);
     BRepMesh_IncrementalMesh(myShape,myDeflection);
 
     myX1OnOff = Standard_False;
@@ -158,10 +158,6 @@ case 6: //color
         return;
       }
 
-      const TColgp_Array1OfPnt& Nodes= myT->Nodes();
-
-      const Poly_Array1OfTriangle& triangles = myT->Triangles();
-
       Standard_Integer nnn = myT->NbTriangles(); // nnn : nombre de triangles
       Standard_Integer nt, n1, n2, n3 = 0;// nt : triangle courant
       // ni : sommet i du triangle courant
@@ -171,15 +167,15 @@ case 6: //color
         // triangles(nt).Get(n1,n2,n3); // le triangle est n1,n2,n3
 
         if (myFace.Orientation() == TopAbs_REVERSED) // si la face est "reversed"
-          triangles(nt).Get(n1,n3,n2); // le triangle est n1,n3,n2
+          myT->Triangle (nt).Get (n1,n3,n2); // le triangle est n1,n3,n2
         else
-          triangles(nt).Get(n1,n2,n3); // le triangle est n1,n2,n3
+          myT->Triangle (nt).Get (n1,n2,n3); // le triangle est n1,n2,n3
 
-        if (TriangleIsValid (Nodes(n1),Nodes(n2),Nodes(n3)) )
+        if (TriangleIsValid (myT->Node (n1), myT->Node (n2), myT->Node (n3)) )
         { // Associates a vertexNT to each node
-          gp_Pnt p = Nodes(n1).Transformed(myLocation.Transformation());
-          gp_Pnt q = Nodes(n2).Transformed(myLocation.Transformation());
-          gp_Pnt r = Nodes(n3).Transformed(myLocation.Transformation());
+          gp_Pnt p = myT->Node (n1).Transformed (myLocation.Transformation());
+          gp_Pnt q = myT->Node (n2).Transformed (myLocation.Transformation());
+          gp_Pnt r = myT->Node (n3).Transformed (myLocation.Transformation());
 
           if (p.Z() > H.Z()) H=p;
           if (q.Z() > H.Z()) H=q;
@@ -213,12 +209,13 @@ case 6: //color
         return;
       }
       Poly_Connect pc(myT);
-      const TColgp_Array1OfPnt& Nodes= myT->Nodes();
+      TColgp_Array1OfPnt Nodes (1, myT->NbNodes());
+      for (Standard_Integer in = 1; in <= myT->NbNodes(); in++)
+      {
+        Nodes.SetValue(in, myT->Node (in));
+      }
       BAR = GProp_PGProps::Barycentre(Nodes);
 
-
-      //const TColgp_Array1OfPnt2d& UVNodes = myT->UVNodes();
-      const Poly_Array1OfTriangle& triangles = myT->Triangles();
       TColgp_Array1OfDir myNormal(Nodes.Lower(), Nodes.Upper());
 
       StdPrs_ToolTriangulatedShape::Normal(myFace, pc, myNormal);
@@ -238,15 +235,15 @@ case 6: //color
       {
         // triangles(nt).Get(n1,n2,n3); // le triangle est n1,n2,n3
         if (myFace.Orientation() == TopAbs_REVERSED) // si la face est "reversed"
-          triangles(nt).Get(n1,n3,n2); // le triangle est n1,n3,n2
+          myT->Triangle (nt).Get (n1,n3,n2); // le triangle est n1,n3,n2
         else
-          triangles(nt).Get(n1,n2,n3); // le triangle est n1,n2,n3
+          myT->Triangle (nt).Get (n1,n2,n3); // le triangle est n1,n2,n3
 
         if (TriangleIsValid (Nodes(n1),Nodes(n2),Nodes(n3)) )
         { // Associates a vertexNT to each node
-          gp_Pnt p = Nodes(n1).Transformed(myLocation.Transformation());
-          gp_Pnt q = Nodes(n2).Transformed(myLocation.Transformation());
-          gp_Pnt r = Nodes(n3).Transformed(myLocation.Transformation());
+          gp_Pnt p = myT->Node(n1).Transformed(myLocation.Transformation());
+          gp_Pnt q = myT->Node(n2).Transformed(myLocation.Transformation());
+          gp_Pnt r = myT->Node(n3).Transformed(myLocation.Transformation());
         }
       }
 
@@ -258,18 +255,18 @@ case 6: //color
         std::cout << "On traite actuellement le triangle : "<< nt <<"\n";
 #endif
         if (myFace.Orientation() == TopAbs_REVERSED) // si la face est "reversed"
-          triangles(nt).Get(n1,n3,n2); // le triangle est n1,n3,n2
+          myT->Triangle (nt).Get (n1,n3,n2); // le triangle est n1,n3,n2
         else
-          triangles(nt).Get(n1,n2,n3); // le triangle est n1,n2,n3
+          myT->Triangle (nt).Get (n1,n2,n3); // le triangle est n1,n2,n3
 
         if (TriangleIsValid (Nodes(n1),Nodes(n2),Nodes(n3)) )
         { // Associates a vertexNT to each node
 
           TColgp_Array1OfPnt Points(1,3);
 
-          gp_Pnt p = Nodes(n1).Transformed(myLocation.Transformation());
-          gp_Pnt q = Nodes(n2).Transformed(myLocation.Transformation());
-          gp_Pnt r = Nodes(n3).Transformed(myLocation.Transformation());
+          gp_Pnt p = myT->Node(n1).Transformed(myLocation.Transformation());
+          gp_Pnt q = myT->Node(n2).Transformed(myLocation.Transformation());
+          gp_Pnt r = myT->Node(n3).Transformed(myLocation.Transformation());
 
           Points(1).SetCoord(p.X(), p.Y(), p.Z());
           Points(2).SetCoord(q.X(), q.Y(), q.Z());
@@ -281,7 +278,7 @@ case 6: //color
         } // end of "if the triangle is valid
       } // end of the "parcours" of the triangles
 
-      Prs3d_Root::CurrentGroup (aPresentation)->AddPrimitiveArray (aOP);
+      aPresentation->CurrentGroup()->AddPrimitiveArray (aOP);
 
       mygroup->SetGroupPrimitivesAspect(myAspect);
     }// end of the exploration of the shape in faces
@@ -291,15 +288,17 @@ case 6: //color
   }
 }
 
-void User_Cylinder::Compute(const Handle(Prs3d_Projector)& aProjector,
-                            const Handle(Prs3d_Presentation)& aPresentation)
+void User_Cylinder::computeHLR (const Handle(Graphic3d_Camera)& aProjector,
+                                const Handle(TopLoc_Datum3D)& ,
+                                const Handle(Prs3d_Presentation)& aPresentation)
 {
   Handle (Prs3d_Drawer) aDefDrawer = GetContext()->DefaultDrawer();
   if (aDefDrawer->DrawHiddenLine())
     myDrawer->EnableDrawHiddenLine();
   else
     myDrawer->DisableDrawHiddenLine();
-  StdPrs_HLRPolyShape::Add(aPresentation,myShape,myDrawer,aProjector);
+  StdPrs_HLRPolyShape aTool;
+  aTool.ComputeHLR (aPresentation,myShape,myDrawer,aProjector);
 }
 
 void User_Cylinder::ComputeSelection(const Handle(SelectMgr_Selection)& aSelection,
