@@ -26,7 +26,6 @@
 #include <Cocoa_LocalPool.hxx>
 
 #include <Image_AlienPixMap.hxx>
-#include <Aspect_Convert.hxx>
 #include <Aspect_WindowDefinitionError.hxx>
 
 IMPLEMENT_STANDARD_RTTIEXT(Cocoa_Window,Aspect_Window)
@@ -111,7 +110,7 @@ Cocoa_Window::Cocoa_Window (const Standard_CString theTitle,
   myYBottom (thePxTop + thePxHeight)
 {
 #if defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE
-  //
+  (void )theTitle;
 #else
   if (thePxWidth <= 0 || thePxHeight <= 0)
   {
@@ -138,6 +137,8 @@ Cocoa_Window::Cocoa_Window (const Standard_CString theTitle,
   {
     throw Aspect_WindowDefinitionError("Unable to create window");
   }
+  // for the moment, OpenGL renderer is expected to output sRGB colorspace
+  [myHWindow setColorSpace: [NSColorSpace sRGBColorSpace]];
   myHView = [[myHWindow contentView] retain];
 
   NSString* aTitleNs = [[NSString alloc] initWithUTF8String: theTitle];
@@ -291,7 +292,7 @@ void Cocoa_Window::Unmap() const
 // function : DoResize
 // purpose  :
 // =======================================================================
-Aspect_TypeOfResize Cocoa_Window::DoResize() const
+Aspect_TypeOfResize Cocoa_Window::DoResize()
 {
   if (myHView == NULL)
   {
@@ -324,10 +325,10 @@ Aspect_TypeOfResize Cocoa_Window::DoResize() const
     default: break;
   }
 
-  *((Standard_Integer* )&myXLeft   ) = (Standard_Integer )aBounds.origin.x;
-  *((Standard_Integer* )&myXRight  ) = (Standard_Integer )(aBounds.origin.x + aBounds.size.width);
-  *((Standard_Integer* )&myYTop    ) = (Standard_Integer )aBounds.origin.y;
-  *((Standard_Integer* )&myYBottom ) = (Standard_Integer )(aBounds.origin.y + aBounds.size.height);
+  myXLeft   = (Standard_Integer )aBounds.origin.x;
+  myXRight  = (Standard_Integer )(aBounds.origin.x + aBounds.size.width);
+  myYTop    = (Standard_Integer )aBounds.origin.y;
+  myYBottom = (Standard_Integer )(aBounds.origin.y + aBounds.size.height);
   return aMode;
 }
 
