@@ -55,7 +55,8 @@
 Extrema_ExtCC::Extrema_ExtCC (const Standard_Real TolC1,
                               const Standard_Real TolC2)
 : myIsFindSingleSolution(Standard_False),
-  myDone (Standard_False)
+  myDone (Standard_False),
+  myIsParallel(Standard_False)
 {
   myC[0] = 0; myC[1] = 0;
   myInf[0] = myInf[1] = -Precision::Infinite();
@@ -111,16 +112,56 @@ Extrema_ExtCC::Extrema_ExtCC(const Adaptor3d_Curve& C1,
   Perform();
 }
 
+//=======================================================================
+//function : Initialize
+//purpose  :
+//=======================================================================
+void Extrema_ExtCC::Initialize (const Adaptor3d_Curve& C1,
+                                const Adaptor3d_Curve& C2,
+                                const Standard_Real TolC1,
+                                const Standard_Real TolC2)
+{
+  // myECC will be re-initialized by Perform()
+  myDone = Standard_False;
+  SetCurve (1, C1, C1.FirstParameter(), C1.LastParameter());
+  SetCurve (2, C2, C2.FirstParameter(), C2.LastParameter());
+  SetTolerance (1, TolC1);
+  SetTolerance (2, TolC2);
+  mydist11 = mydist12 = mydist21 = mydist22 = RealFirst();
+}
+
+//=======================================================================
+//function : Initialize
+//purpose  :
+//=======================================================================
+void Extrema_ExtCC::Initialize (const Adaptor3d_Curve& C1,
+                                const Adaptor3d_Curve& C2,
+                                const Standard_Real U1,
+                                const Standard_Real U2,
+                                const Standard_Real V1,
+                                const Standard_Real V2,
+                                const Standard_Real TolC1,
+                                const Standard_Real TolC2)
+{
+  // myECC will be re-initialized by Perform()
+  myDone = Standard_False;
+  SetCurve (1, C1, U1, U2);
+  SetCurve (2, C2, V1, V2);
+  SetTolerance (1, TolC1);
+  SetTolerance (2, TolC2);
+  mydist11 = mydist12 = mydist21 = mydist22 = RealFirst();
+}
+
 //=======================================================================
 //function : SetCurve
-//purpose  : 
+//purpose  :
 //=======================================================================
 
 void Extrema_ExtCC::SetCurve (const Standard_Integer theRank, const Adaptor3d_Curve& C)
 {
   Standard_OutOfRange_Raise_if (theRank < 1 || theRank > 2, "Extrema_ExtCC::SetCurve()")
   Standard_Integer anInd = theRank - 1;
-  myC[anInd] = (Standard_Address)&C;
+  myC[anInd] = &C;
 }
 
 //=======================================================================
@@ -170,17 +211,16 @@ void Extrema_ExtCC::SetTolerance (const Standard_Integer theRank, const Standard
 void Extrema_ExtCC::Perform()
 {  
   Standard_NullObject_Raise_if (!myC[0] || !myC[1], "Extrema_ExtCC::Perform()")
-  myECC.SetParams(*((Adaptor3d_Curve*)myC[0]), 
-                  *((Adaptor3d_Curve*)myC[1]), myInf[0], mySup[0], myInf[1], mySup[1]);
+  myECC.SetParams(*myC[0], *myC[1], myInf[0], mySup[0], myInf[1], mySup[1]);
   myECC.SetTolerance(Min(myTol[0], myTol[1]));
   myECC.SetSingleSolutionFlag(GetSingleSolutionFlag());
   myDone = Standard_False;
   mypoints.Clear();
   mySqDist.Clear();
-  myIsPar = Standard_False;
+  myIsParallel = Standard_False;
 
-  GeomAbs_CurveType type1 = (*((Adaptor3d_Curve*)myC[0])).GetType();
-  GeomAbs_CurveType type2 = (*((Adaptor3d_Curve*)myC[1])).GetType();
+  GeomAbs_CurveType type1 = myC[0]->GetType();
+  GeomAbs_CurveType type2 = myC[1]->GetType();
   Standard_Real U11, U12, U21, U22, Tol = Min(myTol[0], myTol[1]);
 
   U11 = myInf[0];
@@ -188,20 +228,20 @@ void Extrema_ExtCC::Perform()
   U21 = myInf[1];
   U22 = mySup[1];
 
-  if (!Precision::IsInfinite(U11)) P1f = Extrema_CurveTool::Value(*((Adaptor3d_Curve*)myC[0]),  U11); 
-  if (!Precision::IsInfinite(U12)) P1l = Extrema_CurveTool::Value(*((Adaptor3d_Curve*)myC[0]),  U12);
-  if (!Precision::IsInfinite(U21)) P2f = Extrema_CurveTool::Value(*((Adaptor3d_Curve*)myC[1]), U21);
-  if (!Precision::IsInfinite(U22)) P2l = Extrema_CurveTool::Value(*((Adaptor3d_Curve*)myC[1]), U22);
+  if (!Precision::IsInfinite(U11)) myP1f = Extrema_CurveTool::Value(*myC[0], U11);
+  if (!Precision::IsInfinite(U12)) myP1l = Extrema_CurveTool::Value(*myC[0], U12);
+  if (!Precision::IsInfinite(U21)) myP2f = Extrema_CurveTool::Value(*myC[1], U21);
+  if (!Precision::IsInfinite(U22)) myP2l = Extrema_CurveTool::Value(*myC[1], U22);
   
 
   if (Precision::IsInfinite(U11) || Precision::IsInfinite(U21)) mydist11 = RealLast();
-  else mydist11 = P1f.SquareDistance(P2f);
+  else mydist11 = myP1f.SquareDistance(myP2f);
   if (Precision::IsInfinite(U11) || Precision::IsInfinite(U22)) mydist12 = RealLast();
-  else mydist12 = P1f.SquareDistance(P2l);
+  else mydist12 = myP1f.SquareDistance(myP2l);
   if (Precision::IsInfinite(U12) || Precision::IsInfinite(U21)) mydist21 = RealLast();
-  else mydist21 = P1l.SquareDistance(P2f);
+  else mydist21 = myP1l.SquareDistance(myP2f);
   if (Precision::IsInfinite(U12) || Precision::IsInfinite(U22)) mydist22 = RealLast();
-  else mydist22 = P1l.SquareDistance(P2l);
+  else mydist22 = myP1l.SquareDistance(myP2l);
 
   //Depending on the types of curves, the algorithm is chosen:
   //- _ExtElC, when one of the curves is a line and the other is elementary,
@@ -222,27 +262,27 @@ void Extrema_ExtCC::Perform()
     }
     switch (aType2) {
     case GeomAbs_Line: {
-      Extrema_ExtElC Xtrem((*((Adaptor3d_Curve*)myC[anInd1])).Line(), (*((Adaptor3d_Curve*)myC[anInd2])).Line(), Tol);
+      Extrema_ExtElC Xtrem (myC[anInd1]->Line(), myC[anInd2]->Line(), Tol);
       PrepareResults(Xtrem, isInverse, U11, U12, U21, U22);
       break;
     }
     case GeomAbs_Circle: {
-      Extrema_ExtElC Xtrem((*((Adaptor3d_Curve*)myC[anInd1])).Line(), (*((Adaptor3d_Curve*)myC[anInd2])).Circle(), Tol);
+      Extrema_ExtElC Xtrem (myC[anInd1]->Line(), myC[anInd2]->Circle(), Tol);
       PrepareResults(Xtrem, isInverse, U11, U12, U21, U22);
       break;
     }
     case GeomAbs_Ellipse: {
-      Extrema_ExtElC Xtrem((*((Adaptor3d_Curve*)myC[anInd1])).Line(), (*((Adaptor3d_Curve*)myC[anInd2])).Ellipse());
+      Extrema_ExtElC Xtrem (myC[anInd1]->Line(), myC[anInd2]->Ellipse());
       PrepareResults(Xtrem, isInverse, U11, U12, U21, U22);
       break;
     }
     case GeomAbs_Hyperbola: {
-      Extrema_ExtElC Xtrem((*((Adaptor3d_Curve*)myC[anInd1])).Line(), (*((Adaptor3d_Curve*)myC[anInd2])).Hyperbola());
+      Extrema_ExtElC Xtrem (myC[anInd1]->Line(), myC[anInd2]->Hyperbola());
       PrepareResults(Xtrem, isInverse, U11, U12, U21, U22);
       break;
     }
     case GeomAbs_Parabola: {
-      Extrema_ExtElC Xtrem((*((Adaptor3d_Curve*)myC[anInd1])).Line(), (*((Adaptor3d_Curve*)myC[anInd2])).Parabola());
+      Extrema_ExtElC Xtrem (myC[anInd1]->Line(), myC[anInd2]->Parabola());
       PrepareResults(Xtrem, isInverse, U11, U12, U21, U22);
       break;
     }
@@ -251,7 +291,7 @@ void Extrema_ExtCC::Perform()
   } else if (type1 == GeomAbs_Circle && type2 == GeomAbs_Circle) {
     //analytical case - two circles
     Standard_Boolean bIsDone;
-    Extrema_ExtElC CCXtrem ((*((Adaptor3d_Curve*)myC[0])).Circle(), (*((Adaptor3d_Curve*)myC[1])).Circle());
+    Extrema_ExtElC CCXtrem (myC[0]->Circle(), myC[1]->Circle());
     bIsDone = CCXtrem.IsDone();
     if(bIsDone) {
       PrepareResults(CCXtrem, Standard_False, U11, U12, U21, U22);
@@ -289,7 +329,7 @@ Standard_Boolean Extrema_ExtCC::IsParallel() const
     throw StdFail_NotDone();
   }
 
-  return myIsPar;
+  return myIsParallel;
 }
 
 
@@ -355,10 +395,10 @@ void Extrema_ExtCC::TrimmedSquareDistances(Standard_Real& dist11,
   dist12 = mydist12;
   dist21 = mydist21;
   dist22 = mydist22;
-  P11 = P1f;
-  P12 = P1l;
-  P21 = P2f;
-  P22 = P2l;
+  P11 = myP1f;
+  P12 = myP1l;
+  P21 = myP2f;
+  P22 = myP2l;
 }
 
 //=======================================================================
@@ -371,18 +411,17 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
                                           const Standard_Real theUt22,
                                           const Standard_Real theSqDist)
 {
-  if (!myIsPar)
+  if (!myIsParallel)
     return;
 
-  const GeomAbs_CurveType aType1 = Extrema_CurveTool::GetType(*((Adaptor3d_Curve*) myC[0]));
-  const GeomAbs_CurveType aType2 = Extrema_CurveTool::GetType(*((Adaptor3d_Curve*) myC[1]));
-  
+  const GeomAbs_CurveType aType1 = Extrema_CurveTool::GetType (*myC[0]);
+  const GeomAbs_CurveType aType2 = Extrema_CurveTool::GetType (*myC[1]);
   if (((aType1 != GeomAbs_Line) && (aType1 != GeomAbs_Circle)) ||
       ((aType2 != GeomAbs_Line) && (aType2 != GeomAbs_Circle)))
   {
     mySqDist.Append(theSqDist);
     myDone = Standard_True;
-    myIsPar = Standard_True;
+    myIsParallel = Standard_True;
     return;
   }
   
@@ -393,11 +432,10 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
     //The projection of the circle's location to the trimmed line must exist.
     const Standard_Boolean isReversed = (aType1 != GeomAbs_Circle);
     const gp_Pnt aPonC = !isReversed ?
-                      Extrema_CurveTool::Value(*((Adaptor3d_Curve*) myC[0]), theUt11) :
-                      Extrema_CurveTool::Value(*((Adaptor3d_Curve*) myC[1]), theUt21);
+                      Extrema_CurveTool::Value (*myC[0], theUt11) :
+                      Extrema_CurveTool::Value (*myC[1], theUt21);
 
-    const gp_Lin aL = !isReversed ? ((Adaptor3d_Curve*) myC[1])->Line() :
-                                    ((Adaptor3d_Curve*) myC[0])->Line();
+    const gp_Lin aL = myC[!isReversed ? 1 : 0]->Line();
     const Extrema_ExtPElC ExtPLin(aPonC, aL, Precision::Confusion(),
                                   !isReversed ? theUt21 : theUt11,
                                   !isReversed ? theUt22 : theUt12);
@@ -408,7 +446,7 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
     }
     else
     {
-      myIsPar = Standard_False;
+      myIsParallel = Standard_False;
     }
 
     return;
@@ -435,10 +473,10 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
       // to the 2nd one must intersect the (native) trimmed range of
       // the 2nd line.
 
-      myIsPar = Standard_False;
+      myIsParallel = Standard_False;
 
-      const gp_Lin aLin1 = ((Adaptor3d_Curve*) myC[0])->Line();
-      const gp_Lin aLin2 = ((Adaptor3d_Curve*) myC[1])->Line();
+      const gp_Lin aLin1 = myC[0]->Line();
+      const gp_Lin aLin2 = myC[1]->Line();
       const Standard_Boolean isOpposite(aLin1.Direction().Dot(aLin2.Direction()) < 0.0);
 
       Bnd_Range aRange2(theUt21, theUt22);
@@ -477,7 +515,7 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
       {
         ClearSolutions();
         mySqDist.Append(theSqDist);
-        myIsPar = Standard_True;
+        myIsParallel = Standard_True;
       }
       else if (!aRange2.IsVoid())
       {
@@ -501,12 +539,59 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
         mypoints.Append(aP2);
         mySqDist.Append(theSqDist);
       }
-    }
+      else
+      {
+        //Case like this:
+
+        //  **************     aLin1
+        //                 o
+        //                  o
+        //                   ***********  aLin2
+        // 
+        //Take minimal trimmed distance
+        Standard_Real aDmin, aDists[4] = {mydist11, mydist12, mydist21, mydist22};
+        Extrema_POnCurv aP1, aP2;
+        aDmin = aDists[0];
+        Standard_Integer i, imin = 0;
+        for (i = 1; i < 4; ++i)
+        {
+          if (aDmin > aDists[i])
+          {
+            aDmin = aDists[i];
+            imin = i;
+          }
+        }
+        if (imin == 0)
+        {
+          aP1.SetValues(myInf[0], myP1f);
+          aP2.SetValues(myInf[1], myP2f);
+        }
+        else if (imin == 1)
+        {
+          aP1.SetValues(myInf[0], myP1f);
+          aP2.SetValues(mySup[1], myP2l);
+        }
+        else if (imin == 2)
+        {
+          aP1.SetValues(mySup[0], myP1l);
+          aP2.SetValues(myInf[1], myP2f);
+        }
+        else 
+        {
+          aP1.SetValues(mySup[0], myP1l);
+          aP2.SetValues(mySup[1], myP2l);
+        }
+        ClearSolutions();
+        mypoints.Append(aP1);
+        mypoints.Append(aP2);
+        mySqDist.Append(aDmin);
+      }
+    }   
   }
   else
   {
     // Circle - Circle
-    myIsPar = Standard_False;
+    myIsParallel = Standard_False;
 
     //Two arcs with ranges [U1, U2] and [V1, V2] correspondingly are
     //considered to be parallel in the following case:
@@ -515,12 +600,12 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
     //  the range [V1-PI, V2-PI]. All ranges must be adjusted to correspond
     //  periodic range before checking of intersection.
 
-    const gp_Circ aWorkCirc = ((Adaptor3d_Curve*) myC[1])->Circle();
+    const gp_Circ aWorkCirc = myC[1]->Circle();
     const Standard_Real aPeriod = M_PI + M_PI;
     gp_Vec aVTg1;
     gp_Pnt aP11;
-    const gp_Pnt aP12 = Extrema_CurveTool::Value(*((Adaptor3d_Curve*) myC[0]), theUt12);
-    Extrema_CurveTool::D1(*((Adaptor3d_Curve*) myC[0]), theUt11, aP11, aVTg1);
+    const gp_Pnt aP12 = Extrema_CurveTool::Value (*myC[0], theUt12);
+    Extrema_CurveTool::D1 (*myC[0], theUt11, aP11, aVTg1);
 
     const Bnd_Range aRange(theUt21, theUt22);
     Bnd_Range aProjRng1;
@@ -530,7 +615,7 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
 
     Standard_Real aPar1 = ElCLib::InPeriod(ElCLib::Parameter(aWorkCirc, aP11),
                                            theUt21, theUt21 + aPeriod);
-    const gp_Vec aVTg2 = Extrema_CurveTool::DN(*((Adaptor3d_Curve*) myC[1]), aPar1, 1);
+    const gp_Vec aVTg2 = Extrema_CurveTool::DN (*myC[1], aPar1, 1);
     
     // Check if circles have same/opposite directions
     const Standard_Boolean isOpposite(aVTg1.Dot(aVTg2) < 0.0);
@@ -553,7 +638,7 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
       if ((aRange.Delta() > Precision::Angular()) &&
           ((aPar2 - aPar1) < Precision::Angular()))
       {
-        aPar1 -= aPeriod;
+        aPar2 += aPeriod;
       }
     }
 
@@ -562,11 +647,13 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
 
     Standard_Real aMinSquareDist = RealLast();
 
-    aProjRng1.Add(aPar1 - M_PI);
-    aProjRng1.Add(aPar2 - M_PI);
-    for (Standard_Integer i = 0; i < 2; i++)
+    aProjRng1.Add(aPar1 - aPeriod);
+    aProjRng1.Add(aPar2 - aPeriod);
+    for (Standard_Integer i = 0; i < 3; i++)
     {
-      // Repeat computation twice
+      // Repeat computation three times, shifting the range to PI on each step,
+      // to be able to find if the concentric arcs ranges are intersected in just one parameter
+      // (lower or upper boundary).
 
       Bnd_Range aRng = aProjRng1;
       aRng.Common(aRange);
@@ -574,14 +661,14 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
       //Cases are possible and processed below:
       //1. Extrema does not exist. In this case all common ranges are VOID.
       //2. Arcs are parallel and distance between them is equal to sqrt(theSqDist).
-      //    In this case myIsPar = TRUE definitely.
+      //    In this case myIsParallel = TRUE definitely.
       //3. Arcs are parallel and distance between them is equal to (sqrt(theSqDist) + R),
-      //    where R is the least radius of the both circles. In this case myIsPar flag
+      //    where R is the least radius of the both circles. In this case myIsParallel flag
       //    will temporary be set to TRUE but check will be continued until less
       //    distance will be found. At that, region with the least distance can be
-      //    either a local point or continuous range. In 1st case myIsPar = FALSE and
+      //    either a local point or continuous range. In 1st case myIsParallel = FALSE and
       //    several (or single) extremas will be returned. In the 2nd one
-      //    myIsPar = TRUE and only the least distance will be returned.
+      //    myIsParallel = TRUE and only the least distance will be returned.
       //4. Arcs are not parallel. Then several (or single) extremas will be returned.
 
       if (aRng.Delta() > Precision::Angular())
@@ -590,7 +677,7 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
         aRng.GetIntermediatePoint(0.5, aPar);
         const gp_Pnt aPCirc2 = ElCLib::Value(aPar, aWorkCirc);
         Extrema_ExtPElC ExtPCir(aPCirc2,
-                                Extrema_CurveTool::Circle(*((Adaptor3d_Curve*) myC[0])),
+                                Extrema_CurveTool::Circle (*myC[0]),
                                 Precision::Confusion(), theUt11, theUt12);
 
         Standard_Real aMinSqD = ExtPCir.SquareDistance(1);
@@ -599,11 +686,11 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
           aMinSqD = Min(aMinSqD, ExtPCir.SquareDistance(anExtID));
         }
 
-        if (aMinSqD <= aMinSquareDist)
+        if (aMinSqD <= aMinSquareDist + 10.* Epsilon(1. + aMinSqD))
         {
           ClearSolutions();
           mySqDist.Append(aMinSqD);
-          myIsPar = Standard_True;
+          myIsParallel = Standard_True;
 
           const Standard_Real aDeltaSqDist = aMinSqD - theSqDist;
           const Standard_Real aSqD = Max(aMinSqD, theSqDist);
@@ -642,16 +729,16 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
         const Extrema_POnCurv aP2(aPar, aPCirc2);
 
         Extrema_ExtPElC ExtPCir(aPCirc2,
-                                Extrema_CurveTool::Circle(*((Adaptor3d_Curve*) myC[0])),
+                                Extrema_CurveTool::Circle (*myC[0]),
                                 Precision::Confusion(), theUt11, theUt12);
 
-        Standard_Boolean isFound = !myIsPar;
+        Standard_Boolean isFound = !myIsParallel;
 
         if (!isFound)
         {
-          //If the flag myIsPar was set earlier then it does not mean that
+          //If the flag myIsParallel was set earlier then it does not mean that
           //we have found the minimal distance. Here we check it. If there is
-          //a pair of points, which are in less distance then myIsPar flag
+          //a pair of points, which are in less distance then myIsParallel flag
           //was unset and the algorithm will return these nearest points.
 
           for (Standard_Integer anExtID = 1; anExtID <= ExtPCir.NbExt(); anExtID++)
@@ -667,7 +754,7 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
         if (isFound)
         {
           ClearSolutions();
-          myIsPar = Standard_False;
+          myIsParallel = Standard_False;
           for (Standard_Integer anExtID = 1; anExtID <= ExtPCir.NbExt(); anExtID++)
           {
             mypoints.Append(ExtPCir.Point(anExtID));
@@ -677,7 +764,58 @@ void Extrema_ExtCC::PrepareParallelResult(const Standard_Real theUt11,
           }
         }
       }
+      else
+      {
+        //Case like this:
 
+        //  **************     Cir1
+        //                 o
+        //                  o
+        //                   ***********  Cir2
+        // 
+        //Take minimal trimmed distance
+        myIsParallel = Standard_False;
+        Standard_Real aDmin, aDists[4] = { mydist11, mydist12, mydist21, mydist22 };
+        Extrema_POnCurv aP1, aP2;
+        aDmin = aDists[0];
+        Standard_Integer k, imin = 0;
+        for (k = 1; k < 4; ++k)
+        {
+          if (aDmin > aDists[k])
+          {
+            aDmin = aDists[k];
+            imin = k;
+          }
+        }
+        if (aDmin <= aMinSquareDist + 10.* Epsilon(1. + aDmin))
+        {
+          if (imin == 0)
+          {
+            aP1.SetValues(myInf[0], myP1f);
+            aP2.SetValues(myInf[1], myP2f);
+          }
+          else if (imin == 1)
+          {
+            aP1.SetValues(myInf[0], myP1f);
+            aP2.SetValues(mySup[1], myP2l);
+          }
+          else if (imin == 2)
+          {
+            aP1.SetValues(mySup[0], myP1l);
+            aP2.SetValues(myInf[1], myP2f);
+          }
+          else
+          {
+            aP1.SetValues(mySup[0], myP1l);
+            aP2.SetValues(mySup[1], myP2l);
+          }
+          ClearSolutions();
+          mypoints.Append(aP1);
+          mypoints.Append(aP2);
+          mySqDist.Append(aDmin);
+          aMinSquareDist = Min(aMinSquareDist, aDmin);
+        }
+      }
       aProjRng1.Shift(M_PI);
     }
   }
@@ -701,8 +839,8 @@ void Extrema_ExtCC::PrepareResults(const Extrema_ExtElC&  AlgExt,
 
   myDone = AlgExt.IsDone();
   if (myDone) {
-    myIsPar = AlgExt.IsParallel();
-    if (myIsPar) {
+    myIsParallel = AlgExt.IsParallel();
+    if (myIsParallel) {
       PrepareParallelResult(Ut11, Ut12, Ut21, Ut22, AlgExt.SquareDistance());
     }
     else {
@@ -720,11 +858,13 @@ void Extrema_ExtCC::PrepareResults(const Extrema_ExtElC&  AlgExt,
 	  U = P2.Parameter();
 	}
 
-	if (Extrema_CurveTool::IsPeriodic(*((Adaptor3d_Curve*)myC[0]))) {
-	  U = ElCLib::InPeriod(U, Ut11, Ut11+Extrema_CurveTool::Period(*((Adaptor3d_Curve*)myC[0])));
+	if (Extrema_CurveTool::IsPeriodic (*myC[0]))
+	{
+	  U = ElCLib::InPeriod(U, Ut11, Ut11+Extrema_CurveTool::Period (*myC[0]));
 	}
-	if (Extrema_CurveTool::IsPeriodic(*((Adaptor3d_Curve*)myC[1]))) {
-	  U2 = ElCLib::InPeriod(U2, Ut21, Ut21+Extrema_CurveTool::Period(*((Adaptor3d_Curve*)myC[1])));
+	if (Extrema_CurveTool::IsPeriodic (*myC[1]))
+	{
+	  U2 = ElCLib::InPeriod(U2, Ut21, Ut21+Extrema_CurveTool::Period (*myC[1]));
 	}
 
 	if ((U  >= Ut11 - RealEpsilon())  && 
@@ -772,8 +912,8 @@ void Extrema_ExtCC::PrepareResults(const Extrema_ECC&   AlgExt,
   myDone = AlgExt.IsDone();
   if (myDone)
   {
-    myIsPar = AlgExt.IsParallel();
-    if (myIsPar)
+    myIsParallel = AlgExt.IsParallel();
+    if (myIsParallel)
     {
       PrepareParallelResult(Ut11, Ut12, Ut21, Ut22, AlgExt.SquareDistance());
     }
@@ -787,13 +927,13 @@ void Extrema_ExtCC::PrepareResults(const Extrema_ECC&   AlgExt,
         U2 = P2.Parameter();
 
         // Check points to be into param space.
-        if (Extrema_CurveTool::IsPeriodic(*((Adaptor3d_Curve*) myC[0])))
+        if (Extrema_CurveTool::IsPeriodic (*myC[0]))
         {
-          U = ElCLib::InPeriod(U, Ut11, Ut11 + Extrema_CurveTool::Period(*((Adaptor3d_Curve*) myC[0])));
+          U = ElCLib::InPeriod(U, Ut11, Ut11 + Extrema_CurveTool::Period (*myC[0]));
         }
-        if (Extrema_CurveTool::IsPeriodic(*((Adaptor3d_Curve*) myC[1])))
+        if (Extrema_CurveTool::IsPeriodic (*myC[1]))
         {
-          U2 = ElCLib::InPeriod(U2, Ut21, Ut21 + Extrema_CurveTool::Period(*((Adaptor3d_Curve*) myC[1])));
+          U2 = ElCLib::InPeriod(U2, Ut21, Ut21 + Extrema_CurveTool::Period (*myC[1]));
         }
 
         if ((U >= Ut11 - RealEpsilon()) &&
