@@ -15,7 +15,7 @@
 #include <stdio.h>
 #include <IntPatch_Intersection.hxx>
 
-#include <Adaptor3d_HSurface.hxx>
+#include <Adaptor3d_Surface.hxx>
 #include <Adaptor3d_TopolTool.hxx>
 #include <IntPatch_ALine.hxx>
 #include <IntPatch_ALineToWLine.hxx>
@@ -28,8 +28,8 @@
 
 #include <ProjLib_ProjectOnPlane.hxx>
 #include <Geom_Plane.hxx>
-#include <GeomAdaptor_HSurface.hxx>
-#include <GeomAdaptor_HCurve.hxx>
+#include <GeomAdaptor_Surface.hxx>
+#include <GeomAdaptor_Curve.hxx>
 #include <ProjLib_ProjectedCurve.hxx>
 #include <Geom2dInt_GInter.hxx>
 #include <Geom2dAdaptor_Curve.hxx>
@@ -45,29 +45,39 @@ const IntPatch_SequenceOfLine& IntPatch_Intersection::SequenceOfLine() const { r
 //======================================================================
 IntPatch_Intersection::IntPatch_Intersection ()
  : done(Standard_False),
-   //empt, tgte, oppo,
+   empt(Standard_True),
+   tgte(Standard_False),
+   oppo(Standard_False),
    myTolArc(0.0), myTolTang(0.0),
    myUVMaxStep(0.0), myFleche(0.0),
-   myIsStartPnt(Standard_False)
-   //myU1Start, myV1Start, myU2Start, myV2Start
+   myIsStartPnt(Standard_False),
+   myU1Start(0.0),
+   myV1Start(0.0),
+   myU2Start(0.0),
+   myV2Start(0.0)
 {
 }
 
 //======================================================================
 // function: IntPatch_Intersection
 //======================================================================
-IntPatch_Intersection::IntPatch_Intersection(const Handle(Adaptor3d_HSurface)&  S1,
+IntPatch_Intersection::IntPatch_Intersection(const Handle(Adaptor3d_Surface)&  S1,
                                              const Handle(Adaptor3d_TopolTool)& D1,
-                                             const Handle(Adaptor3d_HSurface)&  S2,
+                                             const Handle(Adaptor3d_Surface)&  S2,
                                              const Handle(Adaptor3d_TopolTool)& D2,
                                              const Standard_Real TolArc,
                                              const Standard_Real TolTang)
  : done(Standard_False),
-   //empt, tgte, oppo,
+   empt(Standard_True),
+   tgte(Standard_False),
+   oppo(Standard_False),
    myTolArc(TolArc), myTolTang(TolTang),
    myUVMaxStep(0.0), myFleche(0.0),
-   myIsStartPnt(Standard_False)
-   //myU1Start, myV1Start, myU2Start, myV2Start
+   myIsStartPnt(Standard_False),
+   myU1Start(0.0),
+   myV1Start(0.0),
+   myU2Start(0.0),
+   myV2Start(0.0)
 {
   if(myTolArc<1e-8) myTolArc=1e-8;
   if(myTolTang<1e-8) myTolTang=1e-8;
@@ -79,16 +89,21 @@ IntPatch_Intersection::IntPatch_Intersection(const Handle(Adaptor3d_HSurface)&
 //======================================================================
 // function: IntPatch_Intersection
 //======================================================================
-IntPatch_Intersection::IntPatch_Intersection(const Handle(Adaptor3d_HSurface)&  S1,
+IntPatch_Intersection::IntPatch_Intersection(const Handle(Adaptor3d_Surface)&  S1,
                                              const Handle(Adaptor3d_TopolTool)& D1,
                                              const Standard_Real TolArc,
                                              const Standard_Real TolTang)
  : done(Standard_False),
-   //empt, tgte, oppo,
+   empt(Standard_True),
+   tgte(Standard_False),
+   oppo(Standard_False),
    myTolArc(TolArc), myTolTang(TolTang),
    myUVMaxStep(0.0), myFleche(0.0),
-   myIsStartPnt(Standard_False)
-   //myU1Start, myV1Start, myU2Start, myV2Start
+   myIsStartPnt(Standard_False),
+   myU1Start(0.0),
+   myV1Start(0.0),
+   myU2Start(0.0),
+   myV2Start(0.0)
 {
   Perform(S1,D1,TolArc,TolTang);
 }
@@ -118,7 +133,7 @@ void IntPatch_Intersection::SetTolerances(const Standard_Real TolArc,
 //======================================================================
 // function: Perform
 //======================================================================
-void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
+void IntPatch_Intersection::Perform(const Handle(Adaptor3d_Surface)&  S1,
                                     const Handle(Adaptor3d_TopolTool)& D1,
                                     const Standard_Real TolArc,
                                     const Standard_Real TolTang)
@@ -148,12 +163,12 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
     {
       gp_Dir aDirection = S1->Direction();
       gp_Ax3 anAxis(gp::Origin(), aDirection);
-      Handle(Adaptor3d_HCurve) aBasisCurve = S1->BasisCurve();
+      Handle(Adaptor3d_Curve) aBasisCurve = S1->BasisCurve();
       ProjLib_ProjectOnPlane Projector(anAxis);
       Projector.Load(aBasisCurve, Precision::Confusion());
-      Handle(GeomAdaptor_HCurve) aProjCurve = Projector.GetResult();
+      Handle(GeomAdaptor_Curve) aProjCurve = Projector.GetResult();
       Handle(Geom_Plane) aPlane = new Geom_Plane(anAxis);
-      Handle(GeomAdaptor_HSurface) aGAHsurf = new GeomAdaptor_HSurface(aPlane);
+      Handle(GeomAdaptor_Surface) aGAHsurf = new GeomAdaptor_Surface(aPlane);
       ProjLib_ProjectedCurve aProjectedCurve(aGAHsurf, aProjCurve);
       Handle(Geom2d_Curve) aPCurve;
       ProjLib::MakePCurveOfType(aProjectedCurve, aPCurve);
@@ -201,10 +216,10 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
 #include <Geom2d_Curve.hxx>
 #include <Geom2dAPI_InterCurveCurve.hxx>
 #include <GeomAdaptor.hxx>
-#include <GeomAdaptor_HCurve.hxx>
+#include <GeomAdaptor_Curve.hxx>
 #include <GeomAdaptor_Curve.hxx>
 #include <GeomAdaptor_Surface.hxx>
-#include <GeomAdaptor_HSurface.hxx>
+#include <GeomAdaptor_Surface.hxx>
 #include <Geom_Plane.hxx>
 #include <ProjLib_ProjectOnPlane.hxx>
 #include <GeomProjLib.hxx>
@@ -219,7 +234,7 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
 //===============================================================
 //function: FUN_GetMinMaxXYZPnt
 //===============================================================
-static void FUN_GetMinMaxXYZPnt( const Handle(Adaptor3d_HSurface)& S,
+static void FUN_GetMinMaxXYZPnt( const Handle(Adaptor3d_Surface)& S,
                                  gp_Pnt& pMin, gp_Pnt& pMax )
 {
   const Standard_Real DU = 0.25 * Abs(S->LastUParameter() - S->FirstUParameter());
@@ -245,13 +260,13 @@ static void FUN_GetMinMaxXYZPnt( const Handle(Adaptor3d_HSurface)& S,
 //==========================================================================
 static void FUN_TrimInfSurf(const gp_Pnt& Pmin,
                             const gp_Pnt& Pmax,
-                            const Handle(Adaptor3d_HSurface)& InfSurf,
+                            const Handle(Adaptor3d_Surface)& InfSurf,
                             const Standard_Real& AlternativeTrimPrm,
-                            Handle(Adaptor3d_HSurface)& TrimS)
+                            Handle(Adaptor3d_Surface)& TrimS)
 {
   Standard_Real TP = AlternativeTrimPrm;
-  Extrema_ExtPS ext1(Pmin, InfSurf->Surface(), 1.e-7, 1.e-7);
-  Extrema_ExtPS ext2(Pmax, InfSurf->Surface(), 1.e-7, 1.e-7);
+  Extrema_ExtPS ext1(Pmin, *InfSurf, 1.e-7, 1.e-7);
+  Extrema_ExtPS ext2(Pmax, *InfSurf, 1.e-7, 1.e-7);
   if(ext1.IsDone() || ext2.IsDone())
   {
     Standard_Real Umax = -1.e+100, Umin = 1.e+100, Vmax = -1.e+100, Vmin = 1.e+100, cU, cV;
@@ -288,7 +303,7 @@ static void FUN_TrimInfSurf(const gp_Pnt& Pmin,
     const Standard_Boolean Usup = Precision::IsPositiveInfinite(InfSurf->LastUParameter());
     const Standard_Boolean Vinf = Precision::IsNegativeInfinite(InfSurf->FirstVParameter()); 
     const Standard_Boolean Vsup = Precision::IsPositiveInfinite(InfSurf->LastVParameter());
-    Handle(Adaptor3d_HSurface) TmpSS;
+    Handle(Adaptor3d_Surface) TmpSS;
     Standard_Integer IsTrimed = 0;
     const Standard_Real tp = 1000.0 * TP;
     if(Vinf && Vsup) { TrimS = InfSurf->VTrim(-tp, tp, 1.0e-7); IsTrimed = 1; }
@@ -452,9 +467,9 @@ static void FUN_GetViso(const Handle(Geom_Surface)& GS,
 //================================================================================
 //function: FUN_PL_Intersection
 //================================================================================
-static void FUN_PL_Intersection(const Handle(Adaptor3d_HSurface)& S1,
+static void FUN_PL_Intersection(const Handle(Adaptor3d_Surface)& S1,
                                 const GeomAbs_SurfaceType&        T1,
-                                const Handle(Adaptor3d_HSurface)& S2,
+                                const Handle(Adaptor3d_Surface)& S2,
                                 const GeomAbs_SurfaceType&        T2,
                                 Standard_Boolean&                 IsOk,
                                 TColgp_SequenceOfPnt&             SP,
@@ -466,8 +481,8 @@ static void FUN_PL_Intersection(const Handle(Adaptor3d_HSurface)& S1,
   Standard_Boolean isoS1isLine[2] = {0, 0};
   Standard_Boolean isoS2isLine[2] = {0, 0};
   Handle(Geom_Curve) C1, C2;
-  const GeomAdaptor_Surface & gas1 = *(GeomAdaptor_Surface*)(&(S1->Surface()));
-  const GeomAdaptor_Surface & gas2 = *(GeomAdaptor_Surface*)(&(S2->Surface()));
+  const GeomAdaptor_Surface & gas1 = *(GeomAdaptor_Surface*)(S1.get());
+  const GeomAdaptor_Surface & gas2 = *(GeomAdaptor_Surface*)(S2.get());
   const Handle(Geom_Surface) gs1 = gas1.Surface();
   const Handle(Geom_Surface) gs2 = gas2.Surface();
   Standard_Real MS1[2], MS2[2];
@@ -658,16 +673,16 @@ static void FUN_NewFirstLast(const GeomAbs_CurveType& ga_ct,
 //================================================================================
 //function: FUN_TrimBothSurf
 //================================================================================
-static void FUN_TrimBothSurf(const Handle(Adaptor3d_HSurface)& S1,
+static void FUN_TrimBothSurf(const Handle(Adaptor3d_Surface)& S1,
                              const GeomAbs_SurfaceType&        T1,
-                             const Handle(Adaptor3d_HSurface)& S2,
+                             const Handle(Adaptor3d_Surface)& S2,
                              const GeomAbs_SurfaceType&        T2,
                              const Standard_Real&              TV,
-                             Handle(Adaptor3d_HSurface)&       NS1,
-                             Handle(Adaptor3d_HSurface)&       NS2)
+                             Handle(Adaptor3d_Surface)&       NS1,
+                             Handle(Adaptor3d_Surface)&       NS2)
 {
-  const GeomAdaptor_Surface & gas1 = *(GeomAdaptor_Surface*)(&(S1->Surface()));
-  const GeomAdaptor_Surface & gas2 = *(GeomAdaptor_Surface*)(&(S2->Surface()));
+  const GeomAdaptor_Surface & gas1 = *(GeomAdaptor_Surface*)(S1.get());
+  const GeomAdaptor_Surface & gas2 = *(GeomAdaptor_Surface*)(S2.get());
   const Handle(Geom_Surface) gs1 = gas1.Surface();
   const Handle(Geom_Surface) gs2 = gas2.Surface();
   const Standard_Real UM1 = 0.5 * (S1->LastUParameter() + S1->FirstUParameter());
@@ -713,7 +728,7 @@ static void FUN_TrimBothSurf(const Handle(Adaptor3d_HSurface)& S1,
   {
     if(TrmV1)
     {
-      Handle(Adaptor3d_HSurface) TS = NS1;
+      Handle(Adaptor3d_Surface) TS = NS1;
       NS1 = TS->UTrim(U1S1, U2S1, 1.0e-7);
     }
     else NS1 = S1->UTrim(U1S1, U2S1, 1.0e-7);
@@ -722,7 +737,7 @@ static void FUN_TrimBothSurf(const Handle(Adaptor3d_HSurface)& S1,
   {
     if(TrmV2)
     {
-      Handle(Adaptor3d_HSurface) TS = NS2;
+      Handle(Adaptor3d_Surface) TS = NS2;
       NS2 = TS->UTrim(U1S2, U2S2, 1.0e-7);
     }
     else NS2 = S2->UTrim(U1S2, U2S2, 1.0e-7);
@@ -733,9 +748,9 @@ static void FUN_TrimBothSurf(const Handle(Adaptor3d_HSurface)& S1,
 //function : Perform
 //purpose  : 
 //=======================================================================
-void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  theS1,
+void IntPatch_Intersection::Perform(const Handle(Adaptor3d_Surface)&  theS1,
                                     const Handle(Adaptor3d_TopolTool)& theD1,
-                                    const Handle(Adaptor3d_HSurface)&  theS2,
+                                    const Handle(Adaptor3d_Surface)&  theS2,
                                     const Handle(Adaptor3d_TopolTool)& theD2,
                                     const Standard_Real TolArc,
                                     const Standard_Real TolTang,
@@ -770,9 +785,9 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  theS1,
     GeomAbs_SurfaceType aCTType;
     Standard_Boolean bToCheck;
     //
-    const Handle(Adaptor3d_HSurface)& aCTSurf = 
+    const Handle(Adaptor3d_Surface)& aCTSurf = 
       (typs1 == GeomAbs_Cone || typs1 == GeomAbs_Torus) ? theS1 : theS2;
-    const Handle(Adaptor3d_HSurface)& aGeomSurf = 
+    const Handle(Adaptor3d_Surface)& aGeomSurf = 
       (typs1 == GeomAbs_Cone || typs1 == GeomAbs_Torus) ? theS2 : theS1;
     //
     aCTType = aCTSurf->GetType();
@@ -991,9 +1006,9 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  theS1,
 //function : Perform
 //purpose  : 
 //=======================================================================
-void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  theS1,
+void IntPatch_Intersection::Perform(const Handle(Adaptor3d_Surface)&  theS1,
                                     const Handle(Adaptor3d_TopolTool)& theD1,
-                                    const Handle(Adaptor3d_HSurface)&  theS2,
+                                    const Handle(Adaptor3d_Surface)&  theS2,
                                     const Handle(Adaptor3d_TopolTool)& theD2,
                                     const Standard_Real TolArc,
                                     const Standard_Real TolTang,
@@ -1029,9 +1044,9 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  theS1,
     GeomAbs_SurfaceType aCTType;
     Standard_Boolean bToCheck;
     //
-    const Handle(Adaptor3d_HSurface)& aCTSurf = 
+    const Handle(Adaptor3d_Surface)& aCTSurf = 
       (typs1 == GeomAbs_Cone || typs1 == GeomAbs_Torus) ? theS1 : theS2;
-    const Handle(Adaptor3d_HSurface)& aGeomSurf = 
+    const Handle(Adaptor3d_Surface)& aGeomSurf = 
       (typs1 == GeomAbs_Cone || typs1 == GeomAbs_Torus) ? theS2 : theS1;
     //
     aCTType = aCTSurf->GetType();
@@ -1228,9 +1243,9 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  theS1,
 //function : ParamParamPerfom
 //purpose  : 
 //=======================================================================
-void IntPatch_Intersection::ParamParamPerfom(const Handle(Adaptor3d_HSurface)&  theS1,
+void IntPatch_Intersection::ParamParamPerfom(const Handle(Adaptor3d_Surface)&  theS1,
                                              const Handle(Adaptor3d_TopolTool)& theD1,
-                                             const Handle(Adaptor3d_HSurface)&  theS2,
+                                             const Handle(Adaptor3d_Surface)&  theS2,
                                              const Handle(Adaptor3d_TopolTool)& theD2,
                                              const Standard_Real TolArc,
                                              const Standard_Real TolTang,
@@ -1258,7 +1273,7 @@ void IntPatch_Intersection::ParamParamPerfom(const Handle(Adaptor3d_HSurface)&
       const Standard_Real MU = Max(Abs(theS2->FirstUParameter()),Abs(theS2->LastUParameter()));
       const Standard_Real MV = Max(Abs(theS2->FirstVParameter()),Abs(theS2->LastVParameter()));
       const Standard_Real AP = Max(MU, MV);
-      Handle(Adaptor3d_HSurface) SS;
+      Handle(Adaptor3d_Surface) SS;
       FUN_TrimInfSurf(pMinXYZ, pMaxXYZ, theS1, AP, SS);
       interpp.Perform(SS,theD1,theS2,theD2,TolTang,TolArc,myFleche,myUVMaxStep);
     }
@@ -1268,7 +1283,7 @@ void IntPatch_Intersection::ParamParamPerfom(const Handle(Adaptor3d_HSurface)&
       const Standard_Real MU = Max(Abs(theS1->FirstUParameter()),Abs(theS1->LastUParameter()));
       const Standard_Real MV = Max(Abs(theS1->FirstVParameter()),Abs(theS1->LastVParameter()));
       const Standard_Real AP = Max(MU, MV);
-      Handle(Adaptor3d_HSurface) SS;
+      Handle(Adaptor3d_Surface) SS;
       FUN_TrimInfSurf(pMinXYZ, pMaxXYZ, theS2, AP, SS);
       interpp.Perform(theS1, theD1, SS, theD2,TolTang, TolArc,myFleche,myUVMaxStep);
     }
@@ -1306,8 +1321,8 @@ void IntPatch_Intersection::ParamParamPerfom(const Handle(Adaptor3d_HSurface)&
     }// 'COLLINEAR LINES'
     else
     {
-      Handle(Adaptor3d_HSurface) nS1 = theS1;
-      Handle(Adaptor3d_HSurface) nS2 = theS2;
+      Handle(Adaptor3d_Surface) nS1 = theS1;
+      Handle(Adaptor3d_Surface) nS2 = theS2;
       FUN_TrimBothSurf(theS1,typs1,theS2,typs2,1.e+8,nS1,nS2);
       interpp.Perform(nS1,theD1,nS2,theD2,TolTang,TolArc,myFleche,myUVMaxStep);
     }// 'NON - COLLINEAR LINES'
@@ -1337,9 +1352,9 @@ void IntPatch_Intersection::ParamParamPerfom(const Handle(Adaptor3d_HSurface)&
 ////function : GeomGeomPerfom
 //purpose  : 
 //=======================================================================
-void IntPatch_Intersection::GeomGeomPerfom(const Handle(Adaptor3d_HSurface)& theS1,
+void IntPatch_Intersection::GeomGeomPerfom(const Handle(Adaptor3d_Surface)& theS1,
                                            const Handle(Adaptor3d_TopolTool)& theD1,
-                                           const Handle(Adaptor3d_HSurface)& theS2,
+                                           const Handle(Adaptor3d_Surface)& theS2,
                                            const Handle(Adaptor3d_TopolTool)& theD2,
                                            const Standard_Real TolArc,
                                            const Standard_Real TolTang,
@@ -1468,9 +1483,9 @@ void IntPatch_Intersection::GeomGeomPerfom(const Handle(Adaptor3d_HSurface)& the
 //purpose  : 
 //=======================================================================
 void IntPatch_Intersection::
-  GeomParamPerfom(const Handle(Adaptor3d_HSurface)&  theS1,
+  GeomParamPerfom(const Handle(Adaptor3d_Surface)&  theS1,
                   const Handle(Adaptor3d_TopolTool)& theD1,
-                  const Handle(Adaptor3d_HSurface)&  theS2,
+                  const Handle(Adaptor3d_Surface)&  theS2,
                   const Handle(Adaptor3d_TopolTool)& theD2,
                   const Standard_Boolean isNotAnalitical,
                   const GeomAbs_SurfaceType typs1,
@@ -1512,8 +1527,8 @@ void IntPatch_Intersection::
     }
     else
     {
-      Handle(Adaptor3d_HSurface) nS1 = theS1;
-      Handle(Adaptor3d_HSurface) nS2 = theS2;
+      Handle(Adaptor3d_Surface) nS1 = theS1;
+      Handle(Adaptor3d_Surface) nS2 = theS2;
       FUN_TrimBothSurf(theS1,typs1,theS2,typs2,1.e+5,nS1,nS2);
       interip.Perform(nS1,theD1,nS2,theD2,myTolArc,myTolTang,myFleche,myUVMaxStep);
     }
@@ -1547,9 +1562,9 @@ void IntPatch_Intersection::
   }
 }
 
-void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
+void IntPatch_Intersection::Perform(const Handle(Adaptor3d_Surface)&  S1,
                                     const Handle(Adaptor3d_TopolTool)& D1,
-                                    const Handle(Adaptor3d_HSurface)&  S2,
+                                    const Handle(Adaptor3d_Surface)&  S2,
                                     const Handle(Adaptor3d_TopolTool)& D2,
                                     const Standard_Real U1,
                                     const Standard_Real V1,
@@ -1561,14 +1576,14 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
   myTolArc = TolArc;
   myTolTang = TolTang;
   if(myFleche == 0.0) {
-#if DEBUG
+#ifdef OCCT_DEBUG
     //std::cout<<" -- IntPatch_Intersection::myFleche fixe par defaut a 0.01 --"<<std::endl;
     //std::cout<<" -- Utiliser la Methode SetTolerances( ... ) "<<std::endl;
 #endif
     myFleche = 0.01;
   }
   if(myUVMaxStep==0.0) {
-#if DEBUG
+#ifdef OCCT_DEBUG
     //std::cout<<" -- IntPatch_Intersection::myUVMaxStep fixe par defaut a 0.01 --"<<std::endl;
     //std::cout<<" -- Utiliser la Methode SetTolerances( ... ) "<<std::endl;
 #endif
@@ -1638,8 +1653,8 @@ void IntPatch_Intersection::Perform(const Handle(Adaptor3d_HSurface)&  S1,
 
 #ifdef DUMPOFIntPatch_Intersection
 
-void IntPatch_Intersection__MAJ_R(Handle(Adaptor2d_HCurve2d) *R1,
-                                  Handle(Adaptor2d_HCurve2d) *,
+void IntPatch_Intersection__MAJ_R(Handle(Adaptor2d_Curve2d) *R1,
+                                  Handle(Adaptor2d_Curve2d) *,
                                   int *NR1,
                                   int *,
                                   Standard_Integer nbR1,
@@ -1664,9 +1679,9 @@ void IntPatch_Intersection__MAJ_R(Handle(Adaptor2d_HCurve2d) *R1,
 #endif
 
 void IntPatch_Intersection::Dump(const Standard_Integer /*Mode*/,
-                                 const Handle(Adaptor3d_HSurface)&  /*S1*/,
+                                 const Handle(Adaptor3d_Surface)&  /*S1*/,
                                  const Handle(Adaptor3d_TopolTool)& /*D1*/,
-                                 const Handle(Adaptor3d_HSurface)&  /*S2*/,
+                                 const Handle(Adaptor3d_Surface)&  /*S2*/,
                                  const Handle(Adaptor3d_TopolTool)& /*D2*/) const 
 { 
 #ifdef DUMPOFIntPatch_Intersection
@@ -1675,7 +1690,7 @@ void IntPatch_Intersection::Dump(const Standard_Integer /*Mode*/,
   //--  construction de la liste des restrictions & vertex 
   //--
   int NR1[MAXR],NR2[MAXR];
-  Handle(Adaptor2d_HCurve2d) R1[MAXR],R2[MAXR];
+  Handle(Adaptor2d_Curve2d) R1[MAXR],R2[MAXR];
   Standard_Integer nbR1=0,nbR2=0;
   for(D1->Init();D1->More() && nbR1<MAXR; D1->Next()) { 
     R1[nbR1]=D1->Value(); 
