@@ -25,9 +25,12 @@
 #include <BRepTools.hxx>
 #include <BRepTools_WireExplorer.hxx>
 #include <GCPnts_TangentialDeflection.hxx>
-#include <Geom_Circle.hxx>
 #include <GeomAbs_SurfaceType.hxx>
 #include <GeomAdaptor_Curve.hxx>
+#include <Geom_ConicalSurface.hxx>
+#include <Geom_CylindricalSurface.hxx>
+#include <Geom_Plane.hxx>
+#include <Geom_SphericalSurface.hxx>
 #include <gp_Circ.hxx>
 #include <Poly_Array1OfTriangle.hxx>
 #include <Poly_Polygon3D.hxx>
@@ -37,11 +40,13 @@
 #include <Select3D_SensitiveBox.hxx>
 #include <Select3D_SensitiveCircle.hxx>
 #include <Select3D_SensitiveCurve.hxx>
+#include <Select3D_SensitiveCylinder.hxx>
 #include <Select3D_SensitiveEntity.hxx>
 #include <Select3D_SensitiveFace.hxx>
 #include <Select3D_SensitiveGroup.hxx>
 #include <Select3D_SensitivePoint.hxx>
 #include <Select3D_SensitiveSegment.hxx>
+#include <Select3D_SensitiveSphere.hxx>
 #include <Select3D_SensitiveTriangle.hxx>
 #include <Select3D_SensitiveTriangulation.hxx>
 #include <Select3D_SensitiveWire.hxx>
@@ -113,8 +118,8 @@ void StdSelect_BRepSelectionTool::Load (const Handle(SelectMgr_Selection)& theSe
                                         const Standard_Real theMaxParam)
 {
   Standard_Integer aPriority = (thePriority == -1) ? GetStandardPriority (theShape, theType) : thePriority;
-
-  if( isAutoTriangulation && !BRepTools::Triangulation (theShape, Precision::Infinite()) )
+  if (isAutoTriangulation
+  && !BRepTools::Triangulation (theShape, Precision::Infinite(), true))
   {
     BRepMesh_IncrementalMesh aMesher(theShape, theDeflection, Standard_False, theDeviationAngle);
   }
@@ -193,8 +198,6 @@ void StdSelect_BRepSelectionTool::Load (const Handle(SelectMgr_Selection)& theSe
     const Handle(SelectMgr_EntityOwner)& anOwner = aSelEntIter.Value()->BaseSensitive()->OwnerId();
     anOwner->SetSelectable (theSelectableObj);
   }
-
-  PreBuildBVH (theSelection);
 }
 
 //==================================================
@@ -269,6 +272,142 @@ void StdSelect_BRepSelectionTool::ComputeSensitive (const TopoDS_Shape& theShape
     {
       TopTools_IndexedMapOfShape aSubfacesMap;
       TopExp::MapShapes (theShape, TopAbs_FACE, aSubfacesMap);
+      if (aSubfacesMap.Extent() == 2) // detect cone
+      {
+        const TopoDS_Face* aFaces[2] =
+        {
+          &TopoDS::Face (aSubfacesMap.FindKey (1)),
+          &TopoDS::Face (aSubfacesMap.FindKey (2))
+        };
+
+        TopLoc_Location aLocSurf;
+        const Handle(Geom_Surface)* aSurfaces[2] =
+        {
+          &BRep_Tool::Surface (*aFaces[0], aLocSurf),
+          &BRep_Tool::Surface (*aFaces[1], aLocSurf)
+        };
+
+        Standard_Integer aConIndex = 0;
+        Handle(Geom_ConicalSurface) aGeomCone = Handle(Geom_ConicalSurface)::DownCast (*aSurfaces[0]);
+        Handle(Geom_Plane) aGeomPln;
+        if (!aGeomCone.IsNull())
+        {
+          aGeomPln = Handle(Geom_Plane)::DownCast (*aSurfaces[1]);
+        }
+        else
+        {
+          aConIndex = 1;
+          aGeomCone = Handle(Geom_ConicalSurface)::DownCast (*aSurfaces[1]);
+          aGeomPln  = Handle(Geom_Plane)::DownCast (*aSurfaces[0]);
+        }
+        if (!aGeomCone.IsNull()
+         && !aGeomPln.IsNull()
+         &&  aGeomPln->Position().Direction().IsEqual (aGeomCone->Position().Direction(), Precision::Angular()))
+        {
+          const gp_Cone aCone = BRepAdaptor_Surface (*aFaces[aConIndex]).Cone();
+          const Standard_Real aRad1 = aCone.RefRadius();
+          const Standard_Real aHeight = (aRad1 != 0.0)
+                                       ? aRad1 / Abs (Tan (aCone.SemiAngle()))
+                                       : aCone.Location().Distance (aGeomPln->Location());
+          const Standard_Real aRad2 = (aRad1 != 0.0) ? 0.0 : Tan (aCone.SemiAngle()) * aHeight;
+          gp_Trsf aTrsf;
+          aTrsf.SetTransformation (aCone.Position(), gp_Ax3());
+          Handle(Select3D_SensitiveCylinder) aSensSCyl = new Select3D_SensitiveCylinder (theOwner, aRad1, aRad2, aHeight, aTrsf);
+          theSelection->Add (aSensSCyl);
+          break;
+        }
+      }
+      if (aSubfacesMap.Extent() == 3) // detect cylinder or truncated cone
+      {
+        const TopoDS_Face* aFaces[3] =
+        {
+          &TopoDS::Face (aSubfacesMap.FindKey (1)),
+          &TopoDS::Face (aSubfacesMap.FindKey (2)),
+          &TopoDS::Face (aSubfacesMap.FindKey (3))
+        };
+
+        TopLoc_Location aLocSurf;
+        const Handle(Geom_Surface)* aSurfaces[3] =
+        {
+          &BRep_Tool::Surface (*aFaces[0], aLocSurf),
+          &BRep_Tool::Surface (*aFaces[1], aLocSurf),
+          &BRep_Tool::Surface (*aFaces[2], aLocSurf)
+        };
+
+        Standard_Integer aConIndex = -1, aNbPlanes = 0;
+        Handle(Geom_ConicalSurface) aGeomCone;
+        Handle(Geom_CylindricalSurface) aGeomCyl;
+        Handle(Geom_Plane) aGeomPlanes[2];
+        for (Standard_Integer aSurfIter = 0; aSurfIter < 3; ++aSurfIter)
+        {
+          const Handle(Geom_Surface)& aSurf = *aSurfaces[aSurfIter];
+          if (aConIndex == -1)
+          {
+            aGeomCone = Handle (Geom_ConicalSurface)::DownCast (aSurf);
+            if (!aGeomCone.IsNull())
+            {
+              aConIndex = aSurfIter;
+              continue;
+            }
+            aGeomCyl = Handle (Geom_CylindricalSurface)::DownCast (aSurf);
+            if (!aGeomCyl.IsNull())
+            {
+              aConIndex = aSurfIter;
+              continue;
+            }
+          }
+          if (aNbPlanes < 2)
+          {
+            aGeomPlanes[aNbPlanes] = Handle(Geom_Plane)::DownCast (aSurf);
+            if (!aGeomPlanes[aNbPlanes].IsNull())
+            {
+              ++aNbPlanes;
+            }
+          }
+        }
+
+        if (!aGeomCone.IsNull())
+        {
+          if (!aGeomPlanes[0].IsNull()
+           && !aGeomPlanes[1].IsNull()
+           &&  aGeomPlanes[0]->Position().Direction().IsEqual (aGeomCone->Position().Direction(), Precision::Angular())
+           &&  aGeomPlanes[1]->Position().Direction().IsEqual (aGeomCone->Position().Direction(), Precision::Angular()))
+          {
+            const gp_Cone aCone = BRepAdaptor_Surface (*aFaces[aConIndex]).Cone();
+            const Standard_Real aRad1 = aCone.RefRadius();
+            const Standard_Real aHeight = aGeomPlanes[0]->Location().Distance (aGeomPlanes[1]->Location());
+            gp_Trsf aTrsf;
+            aTrsf.SetTransformation (aCone.Position(), gp_Ax3());
+            const Standard_Real aTriangleHeight = (aCone.SemiAngle() > 0.0)
+                                                ?  aRad1 / Tan (aCone.SemiAngle())
+                                                :  aRad1 / Tan (Abs (aCone.SemiAngle())) - aHeight;
+            const Standard_Real aRad2 = (aCone.SemiAngle() > 0.0)
+                                       ? aRad1 * (aTriangleHeight + aHeight) / aTriangleHeight
+                                       : aRad1 * aTriangleHeight / (aTriangleHeight + aHeight);
+            Handle(Select3D_SensitiveCylinder) aSensSCyl = new Select3D_SensitiveCylinder (theOwner, aRad1, aRad2, aHeight, aTrsf);
+            theSelection->Add (aSensSCyl);
+            break;
+          }
+        }
+        else if (!aGeomCyl.IsNull())
+        {
+          if (!aGeomPlanes[0].IsNull()
+           && !aGeomPlanes[1].IsNull()
+           &&  aGeomPlanes[0]->Position().Direction().IsEqual (aGeomCyl->Position().Direction(), Precision::Angular())
+           &&  aGeomPlanes[1]->Position().Direction().IsEqual (aGeomCyl->Position().Direction(), Precision::Angular()))
+          {
+            const gp_Cylinder aCyl = BRepAdaptor_Surface (*aFaces[aConIndex]).Cylinder();
+            const Standard_Real aRad = aCyl.Radius();
+            const Standard_Real aHeight = aGeomPlanes[0]->Location().Distance (aGeomPlanes[1]->Location());
+            gp_Trsf aTrsf;
+            aTrsf.SetTransformation (aCyl.Position(), gp_Ax3());
+            Handle(Select3D_SensitiveCylinder) aSensSCyl = new Select3D_SensitiveCylinder (theOwner, aRad, aRad, aHeight, aTrsf);
+            theSelection->Add (aSensSCyl);
+            break;
+          }
+        }
+      }
+
       for (Standard_Integer aShIndex = 1; aShIndex <= aSubfacesMap.Extent(); ++aShIndex)
       {
         ComputeSensitive (aSubfacesMap (aShIndex), theOwner,
@@ -353,7 +492,6 @@ static Handle(TColgp_HArray1OfPnt) GetPointsFromPolygon (const TopoDS_Edge& theE
   if (!anHIndices.IsNull())
   {
     const TColStd_Array1OfInteger& anIndices = anHIndices->Nodes();
-    const TColgp_Array1OfPnt& aNodes = aTriangulation->Nodes();
 
     aResultPoints = new TColgp_HArray1OfPnt (1, anIndices.Length());
 
@@ -361,14 +499,14 @@ static Handle(TColgp_HArray1OfPnt) GetPointsFromPolygon (const TopoDS_Edge& theE
     {
       for (Standard_Integer anIndex (anIndices.Lower()), aPntId (1); anIndex <= anIndices.Upper(); ++anIndex, ++aPntId)
       {
-        aResultPoints->SetValue (aPntId, aNodes (anIndices (anIndex)));
+        aResultPoints->SetValue (aPntId, aTriangulation->Node (anIndices[anIndex]));
       }
     }
     else
     {
       for (Standard_Integer anIndex (anIndices.Lower()), aPntId (1); anIndex <= anIndices.Upper(); ++anIndex, ++aPntId)
       {
-        aResultPoints->SetValue (aPntId, aNodes (anIndices (anIndex)).Transformed (aLocation));
+        aResultPoints->SetValue (aPntId, aTriangulation->Node (anIndices[anIndex]).Transformed (aLocation));
       }
     }
     return aResultPoints;
@@ -448,7 +586,15 @@ void StdSelect_BRepSelectionTool::GetEdgeSensitive (const TopoDS_Shape& theShape
   if (!aPoints.IsNull()
    && !aPoints->IsEmpty())
   {
-    theSensitive = new Select3D_SensitiveCurve (theOwner, aPoints);
+    if (aPoints->Length() == 2)
+    {
+      // don't waste memory, create a segment
+      theSensitive = new Select3D_SensitiveSegment (theOwner, aPoints->First(), aPoints->Last());
+    }
+    else
+    {
+      theSensitive = new Select3D_SensitiveCurve (theOwner, aPoints);
+    }
     return;
   }
 
@@ -474,10 +620,10 @@ void StdSelect_BRepSelectionTool::GetEdgeSensitive (const TopoDS_Shape& theShape
     }
     case GeomAbs_Circle:
     {
-      Handle (Geom_Circle) aCircle = new Geom_Circle (cu3d.Circle());
-      if (aCircle->Radius() <= Precision::Confusion())
+      const gp_Circ aCircle = cu3d.Circle();
+      if (aCircle.Radius() <= Precision::Confusion())
       {
-        theSelection->Add (new Select3D_SensitivePoint (theOwner, aCircle->Location()));
+        theSelection->Add (new Select3D_SensitivePoint (theOwner, aCircle.Location()));
       }
       else
       {
@@ -568,6 +714,35 @@ Standard_Boolean StdSelect_BRepSelectionTool::GetSensitiveForFace (const TopoDS_
   TopLoc_Location aLoc;
   if (Handle(Poly_Triangulation) aTriangulation = BRep_Tool::Triangulation (theFace, aLoc))
   {
+    TopLoc_Location aLocSurf;
+    const Handle(Geom_Surface)& aSurf = BRep_Tool::Surface (theFace, aLocSurf);
+    if (Handle(Geom_SphericalSurface) aGeomSphere = Handle(Geom_SphericalSurface)::DownCast (aSurf))
+    {
+      bool isFullSphere = theFace.NbChildren() == 0;
+      if (theFace.NbChildren() == 1)
+      {
+        const TopoDS_Iterator aWireIter (theFace);
+        const TopoDS_Wire& aWire = TopoDS::Wire (aWireIter.Value());
+        if (aWire.NbChildren() == 4)
+        {
+          Standard_Integer aNbSeamEdges = 0, aNbDegenEdges = 0;
+          for (TopoDS_Iterator anEdgeIter (aWire); anEdgeIter.More(); anEdgeIter.Next())
+          {
+            const TopoDS_Edge& anEdge = TopoDS::Edge (anEdgeIter.Value());
+            aNbSeamEdges += BRep_Tool::IsClosed (anEdge, theFace);
+            aNbDegenEdges += BRep_Tool::Degenerated (anEdge);
+          }
+          isFullSphere = aNbSeamEdges == 2 && aNbDegenEdges == 2;
+        }
+      }
+      if (isFullSphere)
+      {
+        gp_Sphere aSphere = BRepAdaptor_Surface (theFace).Sphere();
+        Handle(Select3D_SensitiveSphere) aSensSphere = new Select3D_SensitiveSphere (theOwner, aSphere.Position().Axis().Location(), aSphere.Radius());
+        theSensitiveList.Append (aSensSphere);
+        return Standard_True;
+      }
+    }
     Handle(Select3D_SensitiveTriangulation) STG = new Select3D_SensitiveTriangulation (theOwner, aTriangulation, aLoc, theInteriorFlag);
     theSensitiveList.Append (STG);
     return Standard_True;
@@ -696,7 +871,7 @@ Standard_Boolean StdSelect_BRepSelectionTool::GetSensitiveForFace (const TopoDS_
             }
             else
             {
-              theSensitiveList.Append (new Select3D_SensitiveCircle (theOwner, new Geom_Circle (cu3d.Circle()), theInteriorFlag, 16));
+              theSensitiveList.Append (new Select3D_SensitiveCircle (theOwner, cu3d.Circle(), theInteriorFlag, 16));
             }
           }
         }
